import argparse

from pypisearch.search import Search


def main() -> None:
    """Main program entrypoint."""

    # Get command arguments
    arg_parser = argparse.ArgumentParser(
        description="Custom pip-search utility by pypi search line"
    )
    arg_parser.add_argument(
        "q", metavar="query", type=str, help="query for search"
    )
    arg_parser.add_argument(
        "-p",
        "--page",
        default="1",
        metavar="page",
        type=str,
        help="search page (default 1, could be range of pages like 1-5)",
    )
    args = arg_parser.parse_args()

    page = args.page

    if "-" in page:
        page_from, page_to = args.page.split("-")
        if page_from > page_to:
            raise ValueError("Page from shouldn't be greater then page to")
    else:
        page_from = page
        page_to = None

    # Parse search url and print result table
    search = Search(query=args.q, page_from=page_from, page_to=page_to)
    print(search.tabulated_result or "No results")


if __name__ == "__main__":
    main()



import requests
from typing import List, Optional

import tabulate

from pypisearch import re_constants as const
from pypisearch.result_item import ResultItem


class Search:
    """Main search process instance."""
    pypi_search_url = "https://pypi.org/search/?q={query}&page={page}"

    def __init__(
        self,
        query: str,
        page_from: str = "",
        page_to: Optional[str] = None,
    ) -> None:
        page_from, page_to = int(page_from), int(page_to) if page_to else None
        self.result = []

        if page_to is None:
            self.result = self.download_data(query=query, page=page_from)
        else:
            for page in range(page_from, page_to + 1):
                result = self.download_data(query=query, page=page)

                if not result:
                    break

                self.result.extend(result)

    def download_data(self, *, query: str, page: int) -> List[ResultItem]:
        url = self.pypi_search_url.format(query=query, page=page)
        page_data = requests.get(url=url).text
        items = const.ITEM_RE.split(page_data)
        result = list(
            filter(
                lambda result_item: not result_item.is_empty,
                map(
                    lambda plain_item: ResultItem(plain_text=plain_item),
                    items,
                ),
            )
        )

        return result

    @property
    def tabulated_result(self) -> str:
        """Returns tabulated list of results."""

        return (
            tabulate.tabulate(
                [
                    [
                        f"{item.name} ({item.version})",
                        f"{item.installed_description}{item.description}",
                    ]
                    for item in self.result
                ],
                tablefmt="plain",
            )
        )
